#encoding: UTF-8
=begin
/***************************************************************************
 *   Copyright © 2008-2014, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

# Completable file-objects can treat their
# contents like a fillable form. Predefined
# fields are searched and replaced by values. 

module Completable

	attr_reader :backup, :summary


	# The main function of this module.  Expects as parameters a Hash of
	# field/substitution- pairs and an optional block.  If a block is
	# given, the original file will be processed and a backup is created.

	def complete(subst_def, &b) 
		@summary = Summary.new(subst_def)
		@field_count = 0
		if(self.respond_to?(:gets) )         
			if(self.respond_to?(:closed?) && !self.closed?)
				self.rewind
				if(subst_def && subst_def.respond_to?(:has_key?))
					cpos = 0
					if(!b)
						@backup="#{self.path}_backup"
						outfile = File.open(@backup, 'w'){|bf| bf.write(self.readlines() )}
						self.rewind
					end
					while(chunk = self.gets())
						chunk, msg = process(chunk, subst_def)
						if(!chunk && msg)
							return nil, msg
						end
						if(b)
							b.call(chunk)
						else
							self.seek(cpos, IO::SEEK_SET)
							self.write(chunk)
						end
						cpos = self.pos
					end
				else
					return nil, 'first parameter must be a Hash'
				end
			else
				return nil, 'completable object must at least be open for reading'
			end
		else
			return nil, 'completable object must also respond to :gets'
		end
		@field_count
	end

	# sets the character sequence, which marks
	# substitutable text 
	def field_delimiter=(d)
		@field_delimiter = d
		@field_regex = Regexp.new("#{d}[^#{d}]*#{d.reverse}")
	end
	# returns the character sequence, which marks
	# substitutable text
	def field_delimiter()
		if(!@field_delimiter)
			field_delimiter=(' ')
		end
		@field_delimiter
	end

	private

	# processes one chunk of text with the given
	# map of field/substitution pairs
	def process(chunk, subst_def)
		if(@field_regex)
			# rpl = chunk.match(@field_regex)
			fields = chunk.scan(@field_regex)
			fields.each do |rpl|
				if rpl
					# offset = rpl.offset(0) if rpl
					offset = chunk.index(rpl)
					test_key = rpl.to_s.delete(@field_delimiter) 
					test_key = test_key.to_sym if test_key
					if(subst_def.has_key?(test_key))
						@summary.add_found(test_key)
						value = subst_def[test_key]
						chunk.gsub!(rpl.to_s, value) if value
						@field_count += 1
					end
				end
			end
			chunk
		else
			return nil, 'field-delimiter must be set.'
		end
	end

	# a class to summarize processing results
	class Summary
		attr_accessor :fields_found 

		def fields_missing 
			if(@fields_found)
				@fields - @fields_found.keys  
			else
				@fields
			end
		end

		def initialize(fields)
			if fields.respond_to?(:keys)
				@fields = Array.new(fields.keys)
			elsif fields.respond_to?(:to_ary)
				@fields = Array.new(fields)
			else
				@fields = Array.new()
			end
			@fields_found = Hash.new
		end

		def add_found(field)
			count = @fields_found[field]
			count = 0 if !count
			@fields_found[field] = count + 1
		end
	end
end

