#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2011-2015 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'logger'

# Creates for each implementing object a member @log and precede its
# output with the name of the class of the object.
module Logging

	@@have_log = false
	@@LOG_CONF = File.dirname(File.absolute_path(__FILE__)) << File::Separator << 'log.conf'

	def init_logger(target = STDOUT, level = Logger::INFO)
		# allow to override the set log-levels with an
		# external configuration (log.conf).
		log_conf
		# Or use the defaults as set here or elsewhere...

		@level ||= level 
		@target ||= target

		@log = Logger.new(@target)
		@log.level = @level

		@log.formatter = proc do |severity, datetime, progname, msg|
			t = Time.now
			"#{self.class.name}: #{severity} #{t.hour}-#{t.min}-#{t.sec}: #{msg}\n"
		end
		if ! @@have_log
			@log.debug self.class.name.dup << ' reading logging-configuration from ' << @@LOG_CONF	
			@@have_log = true
		end

	end

	def log_target=(target)
		@target = target
		@log = Logger.new(@@target)
		@log.level = @level
	end
	def log_level=(level)
		@level = level
		@log.level = @level
	end

	private

	def byte_units(bytes)
		m = bytes / 1000000
		k = (bytes - m) / 1000
		b = bytes - m - k
		
	end

	# Override or set the log-level and target-device, as set in a file 'log.conf'.
	# I do not like the look of this, but it works just the way I want it to.
	# "HEAVANS! Isn't there a standard way to do this in Ruby, for Christ's sake?", you say.
	# Heck, I don't care. <= Read that again, I say.
	def log_conf
		config = level = target = nil
		# puts 'log-config is in ' << @@LOG_CONF
		if(File::exist?(@@LOG_CONF) )
			begin
				conf = File.read(@@LOG_CONF)
				config = instance_eval(conf)
			rescue Exception => ex
				STDERR.puts "WARNING! Cannot evaluate the logger-configuration!" << ' ' << ex.message
				STDERR.puts "Default log-levels apply."
			end
		else
				puts "Default log-levels apply."
		end

		if(config && config.respond_to?(:to_hash) )
			config.default = nil
			config = config[self.class.name.to_sym]
			if(config )
				if(config.respond_to?(:to_ary) && config.size == 2)
					@level, @target = config
					@target.downcase!
					logdir = File.dirname(@target)
					msg = file_check(logdir, :exist?, :directory?, :writable?)
					if(msg) 
						STDERR.puts "WARNING! A logfile for '%s' cannot be written to %s (%s)!" %[self.class.name, logdir, msg] 
						@target = nil
					end
				else
					@level = config
				end	
			end
		end
	end
end
